--//2550EN_06_Code_File.sql: Chapter 6 Code //-- 

-- Run to optimize SQL Server for ad-hoc workloads
USE [master];
GO

EXEC [sp_configure] 'show advanced options', 1;
GO

RECONFIGURE;
GO

EXEC [sp_configure] 'optimize for ad hoc workloads', 1;
GO

RECONFIGURE;
GO

EXEC [sp_configure] 'show advanced options', 0;
GO

RECONFIGURE;
GO

-------------------------------------------------------------

USE [AdventureWorks2012];
GO

--Code snippet #1: Composite index example 
--Examine the following query that returns the list of account numbers and name from Purchasing.Vendor table
--where name and account number starts with character A: 
SELECT  [AccountNumber] ,
        [Name]
FROM    [Purchasing].[Vendor]
WHERE   [AccountNumber] LIKE 'A%'
        AND [Name] LIKE 'A%';
GO

--As our search is based on Name and AccountNumber columns, having the following composite index will improve
--the query execution time significantly:
CREATE NONCLUSTERED INDEX [AK_Vendor _ AccountNumber_Name] 
ON [Purchasing].[Vendor] ([AccountNumber] ASC, [Name] ASC) ON [PRIMARY];
GO

--Code snippet #2: Covering index example
--Consider that you now want to retrieve all columns from the Purchasing.Vendor table, based on values of Name and
--AccountNumber columns. To accomplish this, you execute the following query:
SELECT  [AccountNumber] ,
        [Name] ,
        [CreditRating] ,
        [PreferredVendorStatus] ,
        [ActiveFlag] ,
        [PurchasingWebServiceURL] ,
        [ModifiedDate]
FROM    [Purchasing].[Vendor]
WHERE   [AccountNumber] IN ( N'AUSTRALI0001', N'JEFFSSP0001', N'MITCHELL0001' )
        AND [Name] IN ( N'Inner City Bikes', N'Hill Bicycle Center' );
GO

---To improve this query performance, we can modify our nonclustered composite index on the Purchasing.Vendor table,
--which we create earlier, to add remaining columns of the query as a non-key columns in this composite index:
USE [AdventureWorks2012];
GO

CREATE NONCLUSTERED INDEX [AK_Vendor_AccountNumber_Name] 
ON [Purchasing].[Vendor] ([AccountNumber] ASC, [Name] ASC ) 
INCLUDE([CreditRating]
,[PreferredVendorStatus]
,[ActiveFlag]
,[PurchasingWebServiceURL]
,[ModifiedDate]) ON [PRIMARY];
GO

--Code snippet #3: Filtered index example
--Consider the following query that returns the all orders from Sales.SalesOrderDetail that are placed on 
--or after January, 1st 2008:
SELECT  [SalesOrderID] ,
        [SalesOrderDetailID] ,
        [OrderQty] ,
        [ProductID] ,
        [SpecialOfferID] ,
        [UnitPrice] ,
        [UnitPriceDiscount] ,
        [LineTotal] ,
        [ModifiedDate]
FROM    [Sales].[SalesOrderDetail]
WHERE   [ModifiedDate] >= '2008-01-01 00:00:00.000';
GO

--The following non-clustered filtered index will considerably improve this query performance
CREATE NONCLUSTERED INDEX IXNC_SalesOrderDetail_ModifiedDate 
ON [Sales].[SalesOrderDetail] ([ModifiedDate]) 
INCLUDE ([SalesOrderID]
,[SalesOrderDetailID]
,[OrderQty]
,[ProductID]
,[SpecialOfferID]
,[UnitPrice]
,[UnitPriceDiscount]
,[LineTotal])
WHERE [ModifiedDate] >= '2007-01-01 00:00:00.000';
GO

--Code snippet #4: DBCC SHOW_STATISTICS command to retrieve the header information about the statistics example
--The following DBCC SHOW_STATISTICS command returns statistics information for the AK_Employee_LoginID index of 
--the HumanResources.Employee table in the AdventureWorks2012 database.
DBCC SHOW_STATISTICS (N'HumanResources.Employee', N'AK_Employee_LoginID');
GO

--Code snippet #5: sys.stats catalog view with STATS_DATE() function to retireve date and time when statistics were last updated example
--The following script inserts the data into ProductID column of dbo.CustomProducts table
SELECT  OBJECT_NAME([object_id]) AS [ObjectName] ,
        [name] AS [StatisticName] ,
        STATS_DATE([object_id], [stats_id]) AS [StatisticUpdateDate]
FROM    [sys].[stats];
GO

